/*
Phost Queue Editor (PQueueEd) v1.1a
Copyright 2001,2002,2003 by Piotr Winiarczyk

Basic Phost build queue editor, for VGA-Planets game.

 v1.0  2001-11-06 
      First release.

 v1.1  2001-11-21
      Change to PDK 3.2.4.3 functions.
      
 v1.1a 2003-03-23
      Compiled using gcc 3.2, linked with PDK 4.4
*/

#include "phostpdk.h"

#define  AND   &&
#define  OR    ||
#define  EQ    ==
#define  NEQ   !=
#define  GT    >
#define  GE    >=
#define  LT 	<
#define  LE    =<

#define  MYNAME "PQueueEd"

/* version number */
Uns16 pMajor=1;
Uns16 pMinor=1;
char  pLetter='a';

/* switches */

Uns16 sSort  =False;
Uns16 sList  =True;
Uns16 sBase  =False;
Uns16 sPoints=False;
Uns16 sRace  =False;
Uns16 sAdd   =False;

Uns32 Points=0;
Uns32 Base=0;
int   Add=0;
RaceType_Def Race=0;

static void
version(void)
{
  Info( "VGA-Planets "MYNAME" v%u.%u%c",pMajor, pMinor,pLetter);
}

static void
usage(void)
{
	   Info(
"Usage: "MYNAME" [options] [GameDirectory [RootDirectory]]\n"
"options:\n"
"        -l       -- List queue entries (default action).\n"
"        -s       -- Sort queue entries.\n"
"        -n ID    -- Provide base ID.\n"
"        -p value -- Provide queue points.\n"
"        -r race  -- Specify player number.\n"
"        -a value -- Provide number of priority points to add globaly.\n"
"        -h       -- Print help and exit.\n"
"        -v       -- Print version and exit.\n");
          exit(EXIT_SUCCESS);
}

static void processOptions(int argc, char *argv[])
{
	int c;

    while ((c=pgetopt(argc, argv, "a:r:sn:p:lhHv")) NEQ EOF) {
		switch (c) {
		 case 'h': case 'H': default:
			usage();
                        break;
		 case 'v':
                        Info("PHOST Development Kit Version %u.%u",PDK_VERSION_MAJOR, PDK_VERSION_MINOR);
                        Info("Copyright (C) 1995-2003 Portable Host Project\n");
			version();
			Info("Copyright (C) 2001-2003 Piotr Winiarczyk\n");
			Info("GCC version: %s",__VERSION__);
                        Info("Compilation date & time: %s %s",__DATE__,__TIME__);
                        exit(EXIT_SUCCESS);
		 case 'l':
                     sList=True;
		 break;
		 case 's':
                     sSort=True;
                     sList=False;
		 break;
		 case 'n':
                     sBase=True;
                     sList=False;
                     Base = atoi(poptarg);
		 break;
		 case 'p':
                     sPoints=True;
                     sList=False;
                     Points = atoi(poptarg);
		 break;
		 case 'r':
                     sRace=True;
                     Race = atoi(poptarg);
		 break;
		 case 'a':
                     sAdd=True;
                     sList=False;
                     Add = atoi(poptarg);
		 break;
		}
	}

    if (poptind < argc) {
        gGameDirectory = argv[poptind++];
	}

    if (poptind < argc) {
        gRootDirectory = argv[poptind++];
	}

    if (poptind < argc) usage();
}

int ShowQueue(void)
{
  Uns16 i;
  Uns16 lExit=False;

  for(i=0;i<ShipsInBuildQueue(); i++)
  {

    if ((
       (sRace)&&(BuildQueueOwner(i)==Race))
       ||(!sRace))
    {
    Info("%3d:#%3d:%13s:%5s:%30s:%6d",
                            i+1,
                            BuildQueueBaseID(i),
                            RaceNameAdjective(BuildQueueOwner(i),NULL),
                            (BuildQueueIsCloning(i)?"CLONE":"BUILD"),
                            HullName(BuildQueueHullType(i),NULL),
                            BuildQueuePriority(i));
    lExit=True;
    }
  } // for
  return lExit;
}

int ChangePriority(int lBase,int lPriority)
{
  Uns16 i;
  Uns16 sExit=False;

  for(i=0;i<ShipsInBuildQueue(); i++)
    if (BuildQueueBaseID(i)==lBase)
    {
       PutBuildQueuePriority(i,lPriority);
       sExit=True;
       break;
    }

  return sExit;
}

int AddPriority(int lRace,int lAdd)
{
  Uns16 i;
  Uns16 sExit=False;

  for(i=0;i<ShipsInBuildQueue(); i++)
  {
// race ==0 => no race specific.
    if (lRace==0)
    {
       PutBuildQueuePriority(i,BuildQueuePriority(i)+lAdd);
       sExit=True;
    }
// race != 0
    if (lRace!=0)
    {
 // check for player lRace queue entries
       if (BuildQueueOwner(i)==lRace)
       {
       PutBuildQueuePriority(i,BuildQueuePriority(i)+lAdd);
       sExit=True;
       }
    } // race != 0
  } // for
  return sExit;
}


int main(int argc, char *argv[])
{
   Uns16 sSave=False;
   Uns16 sAction=False;

   gRootDirectory=".";
   gGameDirectory=".";

   processOptions(argc, argv);

   InitPHOSTLib();
   version();
   Info("");

   if (!ReadGlobalData()) ErrorExit("Cannot load global data.");
   if (!ReadHostData())   ErrorExit("Cannot load host data.");

   if ((sRace)&&(!PlayerIsActive(Race)))
     ErrorExit("Specified Player number is invalid : %d",Race);

   if (sList) {
      sAction=ShowQueue();

      if (!sAction)
      if (sRace) Info ("No ships owned by Player %d in queue.",Race);
      else       Info ("No ships in queue.");

      sAction=True;
      sSort=False;
      sAdd=False;
      sBase=False;
      }

   if (sSort) {
     Info ("Sorting...");
     SortBuildQueue();
     sSave=True;
     sAction=True;
     }

   if ((!sSort)&&(sBase)&&(sPoints)&&(!sAdd))
   {
     sSave=ChangePriority(Base,Points);
     if (sSave) Info("Changed priority on base ID:%d to %d.",Base,Points);
     else       Info("ERROR: No ship to build on planet ID:%d",Base);
     sAction=True;
   }

   if ((!sSort)&&(!sBase)&&(!sPoints)&&(sAdd))
   {
     sSave=AddPriority(Race,Add);
     if (sSave)
     {
     if (sRace) Info("Added %d priority points to all player %d queue entries.",Add,Race);
     else       Info("Added %d priority to all queue entries.",Add);
     }
     else       Info("No changes made to priority points.");
     sAction=True;
   }

   if (!sAction)
       ErrorExit("Wrong parameters. Check syntax.");

   if (sSave)
   {
     Info("Saving changes...");
     WriteHostData();
   }

   Info("");
   FreePHOSTLib();
   return EXIT_SUCCESS;
}
